/**
 * This code is released under public domain 
 * by Esa-Petri Tirkkonen
 * no rights reserved
 * */
package com.example.android.snake;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;
import android.view.Display;
import android.view.Surface;
import android.view.WindowManager;

/**
 * @author d0358674
 * 
 */
public class AccelerometerTools implements SensorEventListener {
	private SensorManager mSensorManager;
	private Sensor accelerometer;
	private Sensor compass;
	private Sensor magnet;
	private WindowManager mWindowManager;
	private Display mDisplay;
	private float myAzimuth = 0;

	private Acceleration client;
	
	static final String TAG="AccelerometerTools";

	/**
	 * Constructor need two arguments
	 * 
	 * @param ac
	 *            pointer to class that implements Acceleration
	 * @param activity
	 *            pointer to active Activity
	 */
	public AccelerometerTools(Acceleration ac, Activity activity) {
		// Get an instance of the SensorManager
		mSensorManager = (SensorManager) activity
				.getSystemService(Context.SENSOR_SERVICE);
		// Get an instance of the WindowManager
		mWindowManager = (WindowManager) activity
				.getSystemService(Context.WINDOW_SERVICE);
		mDisplay = mWindowManager.getDefaultDisplay();

		accelerometer = mSensorManager
				.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		
		compass = mSensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION);
		magnet = mSensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
		
		
		
		client=ac;
		Log.d(TAG,"Ready to go");
	}

	/**
	 * Run this to start receiving updates
	 */
	public void StartListening() {
		/*
		 * It is not necessary to get accelerometer events at a very high rate,
		 * by using a slower rate (SENSOR_DELAY_UI), we get an automatic
		 * low-pass filter, which "extracts" the gravity component of the
		 * acceleration. As an added benefit, we use less power and CPU
		 * resources.
		 */
		mSensorManager.registerListener(this, accelerometer,
				SensorManager.SENSOR_DELAY_UI);
		mSensorManager.registerListener(this, compass,
				SensorManager.SENSOR_DELAY_GAME);
//		mSensorManager.registerListener(this, magnet,
//				SensorManager.SENSOR_DELAY_GAME);
		
	}

	/**
	 * run this when done listening
	 */
	public void stopListening() {
		mSensorManager.unregisterListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * android.hardware.SensorEventListener#onAccuracyChanged(android.hardware
	 * .Sensor, int)
	 */
	/**
	 * this is run when accuracy changes
	 */
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * android.hardware.SensorEventListener#onSensorChanged(android.hardware
	 * .SensorEvent)
	 */
	/**
	 * this run every sensor update
	 */
	@TargetApi(8)
	public void onSensorChanged(SensorEvent event) {
		if(event.sensor.getType() == Sensor.TYPE_ORIENTATION) {
			myAzimuth = Math.round(event.values[0]);
			client.sensorUpdate(event.values[0], event.values[1], event.values[2],
					event.timestamp, System.nanoTime(),event.sensor.getType());
			
//			Log.d(TAG,
//					"Azimuth " + Float.toString(event.values[0]) + ","
//							+ Float.toString(event.values[1]) + ","
//							+ Float.toString(event.values[2]));			
			return;
		}				
					
		/*
		 * record the accelerometer data, the event's timestamp as well as the
		 * current time. The latter is needed so we can calculate the "present"
		 * time during rendering. In this application, we need to take into
		 * account how the screen is rotated with respect to the sensors (which
		 * always return data in a coordinate space aligned to with the screen
		 * in its native orientation).
		 */		
		switch (mDisplay.getRotation()) {
		case Surface.ROTATION_0:
			client.sensorUpdate(event.values[0], event.values[1], event.values[2],
					event.timestamp, System.nanoTime(),event.sensor.getType());
			break;
		case Surface.ROTATION_90:
			client.sensorUpdate(-event.values[1], event.values[0],event.values[2],
					event.timestamp, System.nanoTime(),event.sensor.getType());
			break;
		case Surface.ROTATION_180:
			client.sensorUpdate(-event.values[0], -event.values[1],event.values[2],
					event.timestamp, System.nanoTime(),event.sensor.getType());
			break;
		case Surface.ROTATION_270:
			client.sensorUpdate(event.values[1], -event.values[0],event.values[2],
					event.timestamp, System.nanoTime(),event.sensor.getType());
			break;
		}
	}
	
	/**
	 * get maxinum value of accelerometer, values go also this much at negative side
	 * @return 
	 */
	public float getMaximumRange(){
		return this.accelerometer.getMaximumRange();
	}
	
	/**
	 * 
	 * @return accelerometer sensor object
	 */
	public Sensor getAccelerometer() {
		return accelerometer;
	}
	
	public float getMyAzimuth() {
		return myAzimuth;
	}

	public void setMyAzimuth(float myAzimuth) {
		this.myAzimuth = myAzimuth;
	}

}
