package com.example.android.snake.models;

import com.example.android.snake.util.Coordinate;

public abstract class AbstractObject {
	
	private static final int colision_tolerance = 50;	
	
	public AbstractObject(double x, double y, short direction, short speed) {
		super();
		this.x = x;
		this.y = y;
		this.direction = direction;
		this.speed = speed;
	}

	private double x, y;	
	private short direction = 0;
	private short speed = 0;
	private short life = 1;


	public void decreaseLife(){
		if(life > 0)
			life--;
	}
	
	public double distance(double x, double y) {
		double xdiff = this.x - x;
		double ydiff = this.y - y;
		return (double)Math.sqrt(xdiff * xdiff + ydiff * ydiff);
	}
	
	public double distance(AbstractObject that) {		
		return distance(that.x,that.y);
	}

	public Boolean isColided(AbstractObject object2) {
		return this.isColided(object2, this.getColision_tolerance());
	}

	public Boolean isColided(AbstractObject object2, int colision_tolerance) {
		double distance = this.distance(object2);
		if (distance <= colision_tolerance)
			return true;
		else
			return false;
	}

	public void clear() {
		this.x = 0;
		this.y = 0;
	}

	public void moveTo(double x, double y) {
		this.x = x;
		this.y = y;
	}	

	/**
	 * Move to a location using direction and speed.
	 */
	public void move() {
		if(speed == 0)
			return;
		
		update_x();
		update_y();
	}
	
	
	public void update_x() {
		x += (double)Math.cos((Math.PI/180)*direction)*speed/10;
	}
	
	public void rollback_x() {
		x -= (double)Math.cos((Math.PI/180)*direction)*speed/10;
	}
	
	public void update_y() {
		y += (double)Math.sin((Math.PI/180)*direction)*speed/10;
	}
	
	public void rollback_y() {
		y -= (double)Math.sin((Math.PI/180)*direction)*speed/10;
	}
	
	
	public Coordinate getCoordinate() {
		return new Coordinate(x, y, direction);
	}
	
	public abstract int getTileId();

/* --------------> Getters and setters <--------------- */
	public double getX() {
		return x;
	}

	public void setX(int x) {
		this.x = x;
	}

	public double getY() {
		return y;
	}

	public void setY(int y) {
		this.y = y;
	}

	public abstract int getColision_tolerance() ;

	public short getDirection() {
		return direction;
	}

	public void setDirection(short direction) {
		this.direction = direction;
	}

	public short getSpeed() {
		return speed;
	}

	public void setSpeed(short speed) {
		this.speed = speed;
	}

	public short getLife() {
		return life;
	}

	public void setLife(short life) {
		this.life = life;
	}

}
