package com.example.android.snake.models;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import android.util.Log;

public class GameWorld {
	private static final String TAG = "[GameWorld.java]";
	
	Random r = new Random();
	
	
	private Lunatic lunatic;
	private List<Enemy> enemies = new ArrayList<Enemy>(10);;
	private List<Pillow> pillows = new ArrayList<Pillow>(10);;
	private List<AbstractObject> allObjects = new ArrayList<AbstractObject>(30);
	private Map map;
	
	
	private int enemiesLimit = 5; 
	
	
	// world bouds
	private int max_x;
	private int max_y;	
	
	// device screen window offset
	private int offset_x;
	private int offset_y;
	
	private int screen_w;
	private int screen_h;
	
	public GameWorld() {
		map = new Map();
		map.initDefaultMap();
		
		max_x = map.getWorldX_size();
		max_y = map.getWorldY_size();
		
//		Log.d(TAG,"World size: "+max_x+"x"+max_y);
		
		// putting lunatic
		lunatic = new Lunatic(400,400,(short)0,(short)40);
		putObject(lunatic);
	} 
	
		
	public void putObject(AbstractObject o) {
		if(o instanceof Enemy){
			int [] coords = startWave((int)o.getX(), (int)o.getY());
			o.setX(coords[0]);
			o.setY(coords[1]);
			enemies.add((Enemy)o);
		}else if (o instanceof Pillow)
			pillows.add((Pillow)o);
		else if (o instanceof Lunatic)
			lunatic = (Lunatic)o;
					
		allObjects.add(o);
	}
	
	
	public boolean isOutsideWorld(int x, int y) {
		if( x < 0 || x  > map.getWorldX_size() ||
				  y < 0 || y > map.getWorldY_size())
			 return true;
		else
			return false;
	}
	
	public boolean isOnScreen(int x, int y) {
		if( x > offset_x && x  < getScreenMaxX() &&
			y > offset_y && y < getScreenMaxY())
			 return true;
		else
			return false;
	}
	
	public boolean isSomeSpace(int x, int y) {
		return isSomeSpace(x,y,map.getScale());
	}
	
	private boolean isSomeSpace(int x, int y,int tolerance) {
		
		if (isOutsideWorld(x,y))
			return false;
		
		int square = (int) Math.ceil(tolerance/map.getScale());
		
		for (int i = -square; i < square; i++) 
			for (int j = -square; j < square; j++) 
				if(map.isWall(x+i, y+j)) {
					return false;					
				}
		
		for(AbstractObject o: allObjects)
			if(o.distance(x,y) < tolerance)
				return false;
						
		return true;
	}
	
	private boolean isWall(int x, int y,int tolerance) {				
		
		if (isOutsideWorld(x,y))
			return false;
		
		int sqt = (int) ((int) tolerance / Math.sqrt(2));
		if(	map.isWall(x+sqt, y+sqt) || 
			map.isWall(x-sqt, y+sqt) ||
			map.isWall(x+sqt, y-sqt) ||
			map.isWall(x-sqt, y-sqt) ||
			map.isWall(x+tolerance, y) ||
			map.isWall(x-tolerance, y) ||
			map.isWall(x, y-tolerance) ||
			map.isWall(x, y-tolerance)) 
				return false;					
		
						
		return true;
	}

	
	/**
	 * Wave algorithm to find empty space 
	 */
	public int[] startWave(int x, int y) {
		return startWave(x, y, map.getScale());
	}
	
	/**
	 * Wave algorithm to find empty space 
	 */	
	public int[] startWave(int x, int y, int stopTolerance) {		
		int[] arr = {x,y};		
		
		if(isSomeSpace(x, y, stopTolerance)){
			arr[0] =  x;
			arr[1] =  y;
			return arr;
		}
		
		int dx=1;
		int dy=1;
		int cc = 0;
		while (((x - dx) > 0 || (x + dx) < map.getWorldX_size()) && ((y - dy) > 0
				|| (y + dy) < map.getWorldY_size()) && (cc < 10)) {

			if (isSomeSpace(x + dx, y, stopTolerance)) {				
				arr[0] =  x+dx;
				arr[1] =  y;
				return arr;
			}
			if (isSomeSpace(x - dx, y, stopTolerance)) {				
				arr[0] =  x-dx;
				arr[1] =  y;
				return arr;
			}
			if (isSomeSpace(x, y + dy, stopTolerance)) {
				arr[0] =  x;
				arr[1] =  y+dy;				
				return arr;
			}
			if (isSomeSpace(x, y - dy, stopTolerance)) {
				arr[0] =  x;
				arr[1] =  y-dy;
				return arr;
			}
			// LOL!!
			// switch (r.nextInt(4)) {
			// case 0:
			// return startWave(x-dx,y,stopTolerance);
			// case 1:
			// return startWave(x,y-dy,stopTolerance);
			// case 2:
			// return startWave(x,y+dy,stopTolerance);
			// case 3:
			// return startWave(x+dx,y,stopTolerance);
			// default:
			// break;
			dx+=stopTolerance;
			dy+=stopTolerance;
			cc++;
		}			
		
		int[][] coords = {
				{ x + stopTolerance, y },
				{ x - stopTolerance, y },
				{ x , y + stopTolerance},
				{ x , y - stopTolerance}
			};	
		
		
		for (int i = 0; i < 8; i++) {
			int ri = r.nextInt(4);			
			
			if(!isOutsideWorld(coords[ri][0], coords[ri][1])){
//				System.out.println("Starting at ("+coords[ri][0]+","+coords[ri][1]+")");
				arr =	startWave(coords[ri][0], coords[ri][1], stopTolerance);				
				if (arr[0] != x && arr[1] != y)
					return arr;
			}
			
		}
						
		arr[0] =  x;
		arr[1] =  y;
		System.out.println("No free space found!");
		return arr;
	}

	
	public void update() {
		
		List<AbstractObject> toRemove = new ArrayList<AbstractObject>(3);		
		
		// respawn enemies
		if(enemies.size()<enemiesLimit)
			if(r.nextDouble()>0.05) {
				double x = r.nextInt(max_x);
				double y = r.nextInt(max_y);
				
				while (x > offset_x && x < getScreenMaxX()) 
					x = r.nextInt(max_x);
				while (y > offset_y && y < getScreenMaxY()) 
					y = r.nextInt(max_y);
				
				// aim to player
				int direction = direction(x,y,lunatic.getX(),lunatic.getY());
						
				putObject(new Spider(x, y, 
						(short)direction, (short)(lunatic.getSpeed()+15)));
			}
		
		
		for (AbstractObject o : allObjects) {
			o.update_x();
			o.update_y();
			if (o instanceof Pillow){
				for(AbstractObject o_enemy : enemies){
					if (o.isColided(o_enemy)){
						o_enemy.decreaseLife();
						System.out.print("Spider hit!");
						if(0 <= o_enemy.getLife()){
							toRemove.add(o_enemy); //remove spider
						}
						toRemove.add(o); // remove pillow
						System.out.println("Distance: "+o.distance(o_enemy));
						if(allObjects.size() < 1)
							return;
					}
					
				}
				// iterate Enemies
				// if hits, then
					// decrease enemy lives
					// disappear
			}
			
			if (o instanceof Enemy){
				o.setDirection((short)direction(o.getX(),o.getY(),lunatic.getX(),lunatic.getY()));
			}
			if(o instanceof Lunatic){
				for(AbstractObject o_enemy : enemies){
					if (o.isColided(o_enemy)){
						o.decreaseLife();
						if(0 <= o.getLife()){
							// ToDO: end game
						}
						toRemove.add(o_enemy); //remove spider
					}
				}
			}
			
			o.rollback_x();
			o.rollback_y();
		
			o.update_x();
			if (!isWall((int)o.getX(),(int) o.getY(), o.getColision_tolerance())) {
//				Log.d(TAG,o.getClass().getName()+" hits wall on X axis!");				
				if (o instanceof Pillow){
					toRemove.add(o);
				} else {
					o.rollback_x();
				}
				
			}
			
			o.update_y();
			if (!isWall((int)o.getX(),(int) o.getY(), o.getColision_tolerance())) {
//				Log.d(TAG,o.getClass().getName()+" hits wall on Y axis!");
				if (o instanceof Pillow){
					toRemove.add(o);
				} else {
					o.rollback_y();
				}
			}
			
		}
		
		for (AbstractObject o: toRemove) {
			if(enemies.contains(o))
				enemies.remove(o);
			allObjects.remove(o);			
		}
	}
	
	public void updateLunatic(double direction) {
		lunatic.setDirection((short)direction);
	}
	
	
	private int direction(double x0, double y0,double x1, double y1){
		double dx = x0 - x1;
		double dy = y0 - y1;
		int angle;
		
		
		if(dx == 0) {
			if(dy < 0)
				angle = 90;
			else
				angle = 270;
		} else {
			angle = (int) Math.abs((Math.atan(dy / dx)*(180/Math.PI)));			
			if (dy > 0)
				angle += 180;
			if (dy > 0 && dx < 0)
				angle += 90;
		}		
		return angle;
	}
	
	public void throwPillow(int direction) {
		// modify speed
		int speed = 200;
		int diffAngle = direction - lunatic.getDirection(); 
				
		speed += lunatic.getSpeed()*Math.cos((Math.PI/180)*diffAngle) / 2;
		
		putObject(new Pillow(lunatic.getX(), lunatic.getY(), (short)direction, (short)speed));
	}
	
	public void setScreen(int x, int y, int w, int h) {
		offset_x = x;
		offset_y = y;
		screen_w = w;
		screen_h = h;		
	}
	
	private int getScreenMaxX() {
		return offset_x+screen_w;
	}
	
	private int getScreenMaxY() {
		return offset_y+screen_h;
	}
	
	
	public int countEnemiesOnScreen() {
		int res = 0;
		for (Enemy e : enemies) 
			if(isOnScreen((int)e.getX(),(int)e.getY()))
				res++;
//		Log.d(TAG,"Enemies:"+res);
		return res;
	}

	
	////////////////////////////////////////////////////////////////////////////////
	public int getMax_x() {
		return max_x;
	}

	public void setMax_x(int max_x) {
		this.max_x = max_x;
	}

	public int getMax_y() {
		return max_y;
	}

	public void setMax_y(int max_y) {
		this.max_y = max_y;
	}


	public List<AbstractObject> getAllObjects() {
		return allObjects;
	}


	public Map getMap() {
		return map;
	}


	public Lunatic getLunatic() {
		return lunatic;
	}


	public int getEnemiesLimit() {
		return enemiesLimit;
	}


	public void setEnemiesLimit(int enemiesLimit) {
		this.enemiesLimit = enemiesLimit;
	}
	
	
}















































































